// this object reads the entries from a file to arrays. 
// note: persistent!


var scr = "[db_entries:create] ";


was_created = false // will be set to true once all data has been read
if (was_created)
{
	// if we already had read the data, quit the event right here!
	exit;
}

// initializing array for the individual entries
title[0]		= 0
dev[0]			= 0
weight[0]		= 0
line_in_file[0] = 0 // line in the text file

entries			= 0 // total number of entries

data_valid      = true // gets set to falce if ther was an error
error_line      = 0 // if there's an error, this will contain the line number
error_message   = "ERROR:\n"

var current_line = 0
var delim = ","   // csv delimiter, a comma in this case

// load the text file
var fn  = get_open_filename("Text File|*.txt|Comma-separated values|*.csv", "")

if (fn != "")
{
	file_entries = file_text_open_read(fn)
	
	// parse lines in file
	do
	{
		var line = file_text_readln(file_entries);
		var line_length = string_length(line);
		
		// the current line in the text file (for debugging)
		current_line++;
		line_in_file[current_line - 1] = current_line
		
		
		// quick and very dirty data validation by counting delimiters in a  line
		// provided there are no quotation marks in it.
		if (string_count(delim, line) != 2)
		{
			if (string_count("\"", line) == 0)
			{
				// too little or too many commas on a line
				error_message += "Faulty data in line " + string(current_line) + " found:\n"
				                + line + "\n"
				data_valid = false
				break;
			}
			else
			{
				// todo: account for commas inside quoted strings
			}
		}
		

		// game title
		var index = 0;
		var quotesActive = false;
		var currentChar = "";
		var nextChar    = "";
		
		var str = "";
		nextChar = string_char_at(line, ++index);
		do
		{
			currentChar = nextChar;
			if (currentChar == "\"")
			{
				quotesActive = !quotesActive
			}
			else
			{
				str += currentChar;
			}
			nextChar = string_char_at(line, ++index)
			
			//safeguard
			if (index > 1024)
			{
				error_message += "Faulty data for Game Title (perhaps a missing quotation mark?) in line " + string(current_line) + " found:\n"
				                + line + "\n"
				data_valid = false;
				break;
			}
			
		} until ((quotesActive == false) && (nextChar == delim))
		
		str = f_string_trim(str)
		title[current_line - 1] = str
		

		// developer name
		str = "";
		nextChar = string_char_at(line, ++index);
		do
		{
			currentChar = nextChar;
			
			if (currentChar == "\"")
			{
				quotesActive = !quotesActive
			}
			else
			{
				str += currentChar;
			}
			nextChar = string_char_at(line, ++index)
			
			//safeguard
			if (index > 1024)
			{
				error_message += "Faulty data for Developer Name (perhaps a missing quotation mark?) in line " + string(current_line) + " found:\n"
				                + line + "\n"
				data_valid = false;
				break;
			}
		} until ((quotesActive == false) && (nextChar == delim))
		str = f_string_trim(str)
		dev[current_line - 1] = str

		
		// weight for randomness. we don't accept quotes here
		str = "";
		nextChar = string_char_at(line, ++index);
		do
		{
			currentChar = nextChar;
			str += currentChar
			
			nextChar = string_char_at(line, ++index)
		} until ( index >= line_length) 
		str = f_string_trim(str)
		//show_debug_message("\n"+str)
		weight[current_line - 1] = real(string_digits(str))  // conversion of string to real
		
		//data_valid = true
	
	} until (file_text_eof(file_entries))
	
	entries = current_line - 1 // number of entries (starts at 0
	was_created = true; // gets checked on restart, so we don't need to re-read the file every soft-reset
}
else
{
	error_message += "No file containing valid data was selected." + "\n"
	data_valid = false;
}


if (!data_valid)
{
	// bail on error.
	show_message(error_message +"\n\n"+ 
	
	             "The following format is expected:\n"+
	             "A .csv file (UTF-8 preferred) with three columns, separated by commas.\n"+
				 "Cell #1: Name of the game \n"+
				 "Cell #2: Name of the developer \n"+
				 "Cell #3: Weight. The higher, the more likely\n"+
				 "\n"+
				 "If you want to have commas in a cell, you need to wrap them in quotation marks.\n"+
				 "\n"+
				 "Example:\n"+
				 "Tetris, Alexej Pazhitnov, 1\n"+
				 "Super Meat Boy, \"T. Refenes, E. McMillen\", 1\n"+
				 "Super Mario Bros, Nintendo R&D1, 5"
				 
				)
	game_end()
}