var scr = "[obj_wheel:step] ";




if (do_init)
{

    // store position on screen 
	settle_y    = y // starting y
	initial_x   = x
	initial_y   = settle_y - (sprite_h / 2)
	final_y     = settle_y + (sprite_h / 2)
	
    // reset physics starting impulse and friction
	initial_impulse     = random_range(40,50)
	friction_per_second = random_range(7,9)
	
	// reset slot positions
	slot1.x = initial_x
	slot2.x = initial_x
	slot3.x = initial_x
	slot1.y = settle_y - sprite_h
	slot2.y = settle_y
	slot3.y = settle_y + sprite_h


	// the index of each entry as in the array in db_entries for faster access
	slot1_index = -1
	slot2_index = -1
	slot3_index = -1
	
	
	// give the wheel its initial impulse
	velocity = initial_impulse;
	do_update_slots = true;
	
	// populate the three visible slots with random entries
	var rand = -1
	
	
	// slot 1
	rand = f_getWeightedRandom(global.entries.weight)
	slot1_index = rand;
	with (slot1)
	{
		title		   = global.entries.title[rand]
		dev			   = global.entries.dev[rand]
		line_in_file   = global.entries.line_in_file[rand]
	}
	
	// slot 2
	if (first_run)
	{
			slot2.text     = "Press [SPACE] to randomize!"
			first_run = false
	}
	else
	{	
		slot2.text = "" // reset so that it accepts title, dev, etc
		rand = f_getWeightedRandom(global.entries.weight)
		slot2_index = rand;
		with (slot2)
		{
			title		   = global.entries.title[rand]
			dev			   = global.entries.dev[rand]
			line_in_file   = global.entries.line_in_file[rand]
		}
	}
	

	// slot 3
	rand = f_getWeightedRandom(global.entries.weight)
	slot3_index = rand;
	with (slot3)
	{
		title		   = global.entries.title[rand]
		dev			   = global.entries.dev[rand]
		line_in_file   = global.entries.line_in_file[rand]
	}
	
	do_init = false;
}

var delta = (delta_time / 1000000 );

switch (state)
{
	case (STATE.rolling):
	{
		control.winner_line_in_file = -1;
		do_update_slots = true;
        // when a new slot comes into view
		if (y > final_y)
		{
			// count roll overs
			roll_overs++
			//show_debug_message(roll_overs)
			
			
			// get a new random entry
			var rand;
			
			/*
			// the following block was meant to never follow an entry with the
			// same entry again, but that would have skewed the weighted random
			// selection, so this is why it's deactivated
			
			// make sure there are 3 unique entries if there are more than 3 entries in the database!
			if (global.entries.entries >= 2)
			{
				//show_debug_message("ok")
				do
				{
				  rand = f_getWeightedRandom(global.entries.weight)				
				} until (rand != slot1_index 
				      && rand != slot2_index
					  && rand != slot3_index)
			}
			else
			*/
			{
				rand = f_getWeightedRandom(global.entries.weight)	
			}
			
			// transfer the content of the slot items:
			slot3_index = slot2_index
			slot2_index = slot1_index
			slot1_index = rand
			
			slot3.title = slot2.title
			slot2.title = slot1.title
			slot1.title = global.entries.title[rand]
			
			slot3.dev = slot2.dev
			slot2.dev = slot1.dev
			slot1.dev = global.entries.dev[rand]
			
			slot3.line_in_file = slot2.line_in_file
			slot2.line_in_file = slot1.line_in_file
			slot1.line_in_file = global.entries.line_in_file[rand]
			
			
			slot3.state = SLOT_STATE.running
			slot2.state = SLOT_STATE.running
			slot1.state = SLOT_STATE.running
			
			slot3.velocity = velocity
			slot2.velocity = velocity
			slot1.velocity = velocity
			

			// reset the wheel position
			y = initial_y
			
			audio_play_sound(snd_tic,1,0)
		} // end of "if there is a roll-over"
		
		
		// calculate physics
		var fric = delta * friction_per_second;
		velocity -= fric
		y += velocity
		
		// under a certain threshold, decrease the friction to make it more exciting :)
		if (velocity < friction_threshold)
		{
			friction_per_second = friction_2
		}
		
		// if the wheel is about to stop, let's turn it over to the spring solver
		if (velocity <= .1)
		{
			state = STATE.spring_once
		}
	} break;
	
	case (STATE.spring_once):
	{
		// we have decided upon a "winner" and play a sound before handing
		// the animation off to the spring solver
		
		// show which line in the data file was the winning entry so we can remove it
		show_debug_message("Winning entry \"" + slot2.title + "\" was on line " + string(slot2.line_in_file) + ".")
		
		// 2020-04-26
		// Writing the result to an HTML file for display in OBS when streaming
		var win_title    = slot2.title;
		var win_dev      = slot2.dev;
		f_write_to_html_file(win_title, win_dev)
		
		// play the "settled" cue and maybe add some effects here
		audio_play_sound(snd_tada,1,0)
		
		// create effect
		var fx = instance_create_layer(x + sprite_get_width(obj_slot.sprite) / 2, y, "Effects", obj_fx_glow)
		fx.alpha_end     = 0
		fx.fade_duration = 3
		fx.color_start = c_aqua
		fx.color_end = c_aqua
		fx.scale_x_start = 30
		fx.scale_y_start = 30
		fx.scale_x_end = 500
		fx.scale_y_end = 100
		
		state = STATE.spring

	} break;
	
	case (STATE.spring):
	{
		// 1D spring solver
		
		// F = - kx - bv
		// Source: https://www.freebasic.net/forum/viewtopic.php?t=9769
		
		// Force = -stiffness * (length - rest length)
		var distance = (settle_y - y) 
		
		var force = spring_damping * velocity - spring_stiffness * (distance - 0) // rest-length = 0
		
		/*
		Accellerations for each endpoint
		acceleration(endpoint 1) = -Force / mass(endpoint 1)
		acceleration(endpoint 2) = Force / mass(endpoint 2)
		*/
		var acc = -force / spring_mass;
		
		// dv = a * dt
		var delta_velocity = acc * delta
		velocity += delta_velocity	
		
		var delta_y = velocity * delta
		y += delta_y
		
		// if the force is small enough, let's assume the wheel has settled
		if (abs(force) < .5)
		{
			state = STATE.settled_once;
		}
		
		slot3.velocity = velocity
		slot2.velocity = velocity
		slot1.velocity = velocity
		
		show_debug_message("A   " + string(acc))
		show_debug_message("V   " + string(velocity))
		show_debug_message("dV: " + string(delta_velocity))
		show_debug_message("F:  " + string(force))
		show_debug_message("---------------")
		
		
	} break;
	
	case (STATE.settled_once):
	{
		
		// switch the state on the slots
		slot1.state = SLOT_STATE.stopped
		slot2.state = SLOT_STATE.stopped
		slot3.state = SLOT_STATE.stopped
		
		state = STATE.settled
		
		control.spinner_has_settled = true;
		control.winner_line_in_file = slot2.line_in_file;
		
		// play sound
		audio_play_sound(snd_final,1,0)
		
		
	} break;
	
	case (STATE.settled):
	{
		// nothing should move
		do_update_slots = false 
		velocity = 0
		
		slot3.velocity = 0
		slot2.velocity = 0
		slot1.velocity = 0
			
		y = y;
	} break;
} // end of switch state


if (do_update_slots)
{
	// update associated slots
	if (instance_exists(slot1))
		slot1.y = y - sprite_h
	
	if (instance_exists(slot2))
		slot2.y = y

	if (instance_exists(slot3))
		slot3.y = y + sprite_h
}